@@ -1,9 +1,9 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1771 2020-02-25 14:23:23Z willem $
+# $Id: Base.pm 1794 2020-07-03 13:36:51Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1771 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1794 $)[1];
 
 
 #
@@ -26,7 +26,7 @@ our $VERSION = (qw$LastChangedRevision: 1771 $)[1];
 # [Revised March 2016, June 2018]
 
 
-use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.32; 1';
+use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1';
 
 use constant IPv6 => USE_SOCKET_IP;
 
@@ -45,6 +45,7 @@ use strict;
 use warnings;
 use integer;
 use Carp;
+use IO::File;
 use IO::Select;
 use IO::Socket;
 
@@ -188,8 +189,7 @@ sub _read_config_file {			## read resolver config file
 	my $self = shift;
 	my $file = shift;
 
-	my $filehandle;
-	open( $filehandle, '<', $file ) or croak "$file: $!";
+	my $filehandle = new IO::File( $file, '<' ) or croak "$file: $!";
 
 	my @nameserver;
 	my @searchlist;
@@ -508,8 +508,7 @@ NAMESERVER: foreach my $ns (@ns) {
 
 			my $reply;
 			while ( my ($socket) = $select->can_read($timeout) ) {
-				my $peer = $socket->peerhost;
-				$self->{replyfrom} = $peer;
+				my $peer = $self->{replyfrom} = $socket->peerhost;
 
 				my $buffer = _read_udp( $socket, $self->_packetsz );
 				$self->_diag( "reply from [$peer]", length($buffer), 'bytes' );
@@ -719,7 +718,7 @@ sub axfr {				## zone transfer
 			unless ( scalar @rr ) {
 				my $reply;			# refill @rr
 				( $reply, $verify ) = $self->_axfr_next( $select, $verify );
-				@rr = $reply->answer;
+				@rr = $reply->answer if $reply;
 			}
 
 			return $rr;
@@ -801,12 +800,8 @@ sub _axfr_next {
 	my ($socket) = $select->can_read( $self->{tcp_timeout} );
 	croak $self->errorstring('timed out') unless $socket;
 
-	$self->{replyfrom} = $socket->peerhost;
-
 	my $buffer = _read_tcp($socket);
-	$self->_diag( 'received', length($buffer), 'bytes' );
-
-	my $packet = Net::DNS::Packet->new( \$buffer );
+	my $packet = Net::DNS::Packet->decode( \$buffer );
 	croak $@, $self->errorstring('corrupt packet') if $@;
 
 	return ( $packet, $verify ) unless $verify;
@@ -972,11 +967,15 @@ sub _make_query_packet {
 
 	my ($packet) = @_;
 	if ( ref($packet) ) {
+		my $edns = $packet->edns;			# advertise UDPsize for local stack
+		$edns->size( $self->{udppacketsize} ) unless defined $edns->{size};
+
 		my $header = $packet->header;
 		$header->rd( $self->{recurse} ) if $header->opcode eq 'QUERY';
 
 	} else {
 		$packet = Net::DNS::Packet->new(@_);
+		$packet->edns->size( $self->{udppacketsize} );
 
 		my $header = $packet->header;
 		$header->ad( $self->{adflag} );			# RFC6840, 5.7
@@ -985,8 +984,6 @@ sub _make_query_packet {
 		$header->rd( $self->{recurse} );
 	}
 
-	$packet->edns->size( $self->{udppacketsize} );		# advertise UDPsize for local stack
-
 	if ( $self->{tsig_rr} ) {
 		$packet->sign_tsig( $self->{tsig_rr} ) unless $packet->sigrr;
 	}
@@ -1009,25 +1006,25 @@ sub dnssec {
 
 sub force_v6 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{force_v6};
+	my $value = scalar(@_) ? $_[0] : $self->{force_v6};
 	$self->{force_v6} = $value ? do { $self->{force_v4} = 0; 1 } : 0;
 }
 
 sub force_v4 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{force_v4};
+	my $value = scalar(@_) ? $_[0] : $self->{force_v4};
 	$self->{force_v4} = $value ? do { $self->{force_v6} = 0; 1 } : 0;
 }
 
 sub prefer_v6 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{prefer_v6};
+	my $value = scalar(@_) ? $_[0] : $self->{prefer_v6};
 	$self->{prefer_v6} = $value ? do { $self->{prefer_v4} = 0; 1 } : 0;
 }
 
 sub prefer_v4 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{prefer_v4};
+	my $value = scalar(@_) ? $_[0] : $self->{prefer_v4};
 	$self->{prefer_v4} = $value ? do { $self->{prefer_v6} = 0; 1 } : 0;
 }
 
